package com.flyfish.framework.r2dbc.config;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.r2dbc.config.callback.ReferenceR2dbcCallback;
import com.flyfish.framework.r2dbc.operations.R2dbcReactiveEntityOperations;
import com.flyfish.framework.r2dbc.repository.factory.DefaultReactiveRepositoryFactoryBean;
import com.flyfish.framework.r2dbc.repository.impl.DefaultReactiveRepositoryImpl;
import com.flyfish.framework.repository.ReactiveEntityOperations;
import io.r2dbc.spi.ConnectionFactory;
import org.springframework.boot.autoconfigure.AutoConfiguration;
import org.springframework.boot.autoconfigure.r2dbc.R2dbcAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.mapping.callback.EntityCallback;
import org.springframework.data.r2dbc.config.EnableR2dbcAuditing;
import org.springframework.data.r2dbc.core.R2dbcEntityOperations;
import org.springframework.data.r2dbc.repository.config.EnableR2dbcRepositories;
import org.springframework.r2dbc.connection.init.CompositeDatabasePopulator;
import org.springframework.r2dbc.connection.init.ConnectionFactoryInitializer;
import org.springframework.r2dbc.connection.init.DatabasePopulator;
import org.springframework.r2dbc.connection.init.ResourceDatabasePopulator;

import java.util.Optional;

/**
 * r2dbc数据配置
 *
 * @author wangyu
 */
@EnableR2dbcRepositories(
        repositoryFactoryBeanClass = DefaultReactiveRepositoryFactoryBean.class,
        repositoryBaseClass = DefaultReactiveRepositoryImpl.class,
        basePackages = "com.flyfish.framework.*.repository"
)
@EnableR2dbcAuditing
@AutoConfiguration(before = R2dbcAutoConfiguration.class)
public class R2dbcDataConfig {

    @Bean
    public ReactiveEntityOperations r2dbcReactiveEntityOperations(R2dbcEntityOperations r2dbcEntityOperations) {
        return new R2dbcReactiveEntityOperations(r2dbcEntityOperations);
    }

    @Bean
    public EntityCallback<Domain> referenceR2dbcCallback() {
        return new ReferenceR2dbcCallback();
    }

    /**
     * 数据库初始化
     *
     * @param connectionFactory 链接仓库
     * @return 结果
     */
    @Bean
    public ConnectionFactoryInitializer connectionFactoryInitializer(ConnectionFactory connectionFactory) {
        ConnectionFactoryInitializer initializer = new ConnectionFactoryInitializer();
        initializer.setConnectionFactory(connectionFactory);
        CompositeDatabasePopulator populator = new CompositeDatabasePopulator();
        // 添加主要脚本
        preparePopulator("schema.sql").ifPresent(populator::addPopulators);
        // 动态执行方言脚本
        preparePopulator("dialect/" + connectionFactory.getMetadata().getName().toLowerCase() + ".sql")
                .ifPresent(populator::addPopulators);
        initializer.setDatabasePopulator(populator);
        // 添加清理脚本
        preparePopulator("schema-clean.sql").ifPresent(initializer::setDatabaseCleaner);
        // 返回初始化器
        return initializer;
    }

    /**
     * 动态判定存在性，提供健壮的架构
     *
     * @param path 文件路径
     * @return 结果
     */
    private Optional<DatabasePopulator> preparePopulator(String path) {
        ClassPathResource schema = new ClassPathResource(path);
        if (schema.exists()) {
            return Optional.of(new ResourceDatabasePopulator(schema));
        }
        return Optional.empty();
    }
}
